/*
*****************************************************************

Library Name        : RF12 Transceiver Library

Release Date        : 30/12/2010

Library Author      : Nart Schinackow 

Library Version     : V.1.0

Programing Language : C

Notes               : This Library Was Tested And Working 100% Fine, It Is Under Your Responsibility To Add Or Remove Any Of The Code

Library Functions   :
void Init_Rf12Tx();
void Init_Rf12Rx();
void Write_Zero();
void Write_One();
void Write_Command(unsigned int CMD);
void Rf12_SendByte(unsigned char DATA);
unsigned char Rf12_ReceiveByte();
unsigned char ReceivePacket();
unsigned char ReceiveTimedPacket(unsigned int TimeOut);
void SendPacket(char* DataBytes);
*****************************************************************
*/

extern sfr sbit Rf12_Data_In;
extern sfr sbit Rf12_Data_Out;
extern sfr sbit Rf12_Clock;
extern sfr sbit Rf12_NSEL;
extern sfr sbit Rf12_NIRQ;
extern sfr sbit Rf12_Tx_Led;
extern sfr sbit Rf12_Rx_Led;

extern sfr sbit Rf12_Data_In_Direction;
extern sfr sbit Rf12_Data_Out_Direction;
extern sfr sbit Rf12_Clock_Direction;
extern sfr sbit Rf12_NSEL_Direction;
extern sfr sbit Rf12_NIRQ_Direction;
extern sfr sbit Rf12_Tx_Led_Direction;
extern sfr sbit Rf12_Rx_Led_Direction;

char ReceiveBuffer[22];

void BlinkTxRxLeds(unsigned char NumberOfBlinks, unsigned int BlinkMs)
{
unsigned char i;
for(i=1;i<=NumberOfBlinks;i++)
{
Rf12_Tx_Led=1;
Rf12_Rx_Led=0;
VDelay_ms(BlinkMs);
Rf12_Tx_Led=0;
Rf12_Rx_Led=1;
VDelay_ms(BlinkMs);
}
Rf12_Tx_Led=0;
Rf12_Rx_Led=0;
}


void Write_One()
{
Rf12_Data_In=1;
Rf12_Clock=0;
asm nop;
asm nop;
asm nop;
asm nop;
asm nop;
asm nop;
asm nop;
asm nop;
asm nop;
asm nop;
asm nop;
asm nop;
asm nop;
asm nop;
asm nop;
asm nop;
Rf12_Clock=1;
asm nop;
}

void Write_Zero()
{
Rf12_Data_In=0;
Rf12_Clock=0;
asm nop;
asm nop;
asm nop;
asm nop;
asm nop;
asm nop;
asm nop;
asm nop;
asm nop;
asm nop;
asm nop;
asm nop;
asm nop;
asm nop;
asm nop;
asm nop;
Rf12_Clock=1;
asm nop;
}


void Write_Command(unsigned int CMD)
{
unsigned char n;
n=16;
Rf12_Clock=0;
Rf12_NSEL=0;
while(n--)
{
if(CMD&0x8000)Write_One();
else Write_Zero();
CMD=CMD<<1;
}
Rf12_Clock=0;
Rf12_NSEL=1;
}

void Init_Rf12Tx()
{
Rf12_NSEL_Direction=0;
Rf12_Data_In_Direction=0;
Rf12_Data_Out_Direction=1;
Rf12_Clock_Direction=0;
Rf12_NIRQ_Direction=1;
Rf12_Tx_Led_Direction=0;
Rf12_Rx_Led_Direction=0;


Rf12_NSEL=1;
Rf12_Data_In=1;
Rf12_Clock=0;
Write_Command(0x80D8);
Write_Command(0x8208);
Write_Command(0xA640);
Write_Command(0xC647);
Write_Command(0x94A0);
Write_Command(0xC2AC);
Write_Command(0xCA80);
Write_Command(0xCA83);
Write_Command(0xC49B);
Write_Command(0x9850);
Write_Command(0xE000);
Write_Command(0xC80E);
Write_Command(0xC000);
}

void Init_Rf12Rx()
{
Rf12_NSEL_Direction=0;
Rf12_Data_In_Direction=0;
Rf12_Data_Out_Direction=1;
Rf12_Clock_Direction=0;
Rf12_NIRQ_Direction=1;
Rf12_Tx_Led_Direction=0;
Rf12_Rx_Led_Direction=0;


Rf12_NSEL=1;
Rf12_Data_In=1;
Rf12_Clock=0;
Rf12_Data_Out=0;

Write_Command(0x80D8);
Write_Command(0x82D8);//*
Write_Command(0xA640);
Write_Command(0xC647);
Write_Command(0x94A0);
Write_Command(0xC2AC);
Write_Command(0xCA80);
Write_Command(0xCA83);
Write_Command(0xC49B);
Write_Command(0x9850);
Write_Command(0xE000);
Write_Command(0xC800);//*
Write_Command(0xC000);
}

unsigned char Rf12_ReceiveByte()
{
unsigned char i,result;
Rf12_Clock=0;
Rf12_Data_In=0;
Rf12_NSEL=0;
for(i=0;i<16;i++)
{Rf12_Clock=1;
asm nop;
asm nop;
Rf12_Clock=0;
asm nop;
asm nop;}
result=0;
for(i=0;i<8;i++)
{
result=result<<1;
if(Rf12_Data_Out)result|=1;
Rf12_Clock=1;
asm nop;
asm nop;
Rf12_Clock=0;
asm nop;
asm nop;
}
Rf12_NSEL=1;
return result;
}

void Rf12_SendByte(unsigned char DATA)
{
unsigned char RGIT;
unsigned int temp;
RGIT=0;
temp=0xB800;
temp|=DATA;
while(1)
{
Rf12_Clock=0;
Rf12_NSEL=0;
Rf12_Data_In=0;
Rf12_Clock=1;
if(Rf12_Data_Out)RGIT=1;
else RGIT=0;
Rf12_Clock=0;
Rf12_Data_In=1;
Rf12_NSEL=1;
if(RGIT){RGIT=0;Write_Command(temp);break;}
}
}

unsigned char ReceivePacket()
{
unsigned int Crc3;
unsigned int CrcSum;
unsigned char count;
unsigned char sum;
count=0;
CrcSum=0;
Crc3=0;
while(1)
{
while(!Rf12_NIRQ)
{
ReceiveBuffer[count++]=Rf12_ReceiveByte();
if(count==21){
Write_Command(0xCA80);Write_Command(0xCA83);sum=0;Crc3=Crc3|ReceiveBuffer[19];Crc3=(Crc3<<8)|ReceiveBuffer[20];
sum=ReceiveBuffer[0]+ReceiveBuffer[1];
for(count=0;count<16;count++)CrcSum+=ReceiveBuffer[count+2];
if(sum==ReceiveBuffer[18]&&CrcSum==Crc3){StringNCopy(ReceiveBuffer,ReceiveBuffer+2,16);ReceiveBuffer[16]='\0';return 1;}
else return 0;
}
}
}
}

unsigned char ReceiveTimedPacket(unsigned int TimeOut)
{
unsigned int T;
unsigned int Crc3;
unsigned int CrcSum;
unsigned char count;
unsigned char sum;
count=0;
CrcSum=0;
Crc3=0;
T=0;
while(1)
{
while(!Rf12_NIRQ)
{
ReceiveBuffer[count++]=Rf12_ReceiveByte();
if(count==21){
Write_Command(0xCA80);Write_Command(0xCA83);sum=0;Crc3=Crc3|ReceiveBuffer[19];Crc3=(Crc3<<8)|ReceiveBuffer[20];
sum=ReceiveBuffer[0]+ReceiveBuffer[1];
for(count=0;count<16;count++)CrcSum+=ReceiveBuffer[count+2];
if(sum==ReceiveBuffer[18]&&CrcSum==Crc3){StringNCopy(ReceiveBuffer,ReceiveBuffer+2,16);ReceiveBuffer[16]='\0';return 1;}
else return 0;
}
}
++T;
if(T==TimeOut)return 2;
}
}

void SendPacket(char* DataBytes)
{
unsigned char i;
unsigned int crc3;
unsigned char crc3H;
unsigned char crc3L;
crc3=0;
for(i=0;i<16;i++)
crc3+=DataBytes[i];
crc3L=crc3&0x00ff;
crc3H=(crc3>>8)&0x00ff;

Write_Command(0x8228);
delay_us(5);
Write_Command(0x8238);
asm nop;
asm nop;

/*Packet Header     */
Rf12_SendByte(0xAA);
Rf12_SendByte(0xAA);
Rf12_SendByte(0xAA);
Rf12_SendByte(0x2D);
Rf12_SendByte(0xD4);
/*------------------*/

/*CRC 1             */
Rf12_SendByte(0x7A);
Rf12_SendByte(0x7A);
/*------------------*/

/*Packet DataBytes  */
Rf12_SendByte(DataBytes[0]);
Rf12_SendByte(DataBytes[1]);
Rf12_SendByte(DataBytes[2]);
Rf12_SendByte(DataBytes[3]);
Rf12_SendByte(DataBytes[4]);
Rf12_SendByte(DataBytes[5]);
Rf12_SendByte(DataBytes[6]);
Rf12_SendByte(DataBytes[7]);
Rf12_SendByte(DataBytes[8]);
Rf12_SendByte(DataBytes[9]);
Rf12_SendByte(DataBytes[10]);
Rf12_SendByte(DataBytes[11]);
Rf12_SendByte(DataBytes[12]);
Rf12_SendByte(DataBytes[13]);
Rf12_SendByte(DataBytes[14]);
Rf12_SendByte(DataBytes[15]);
/*------------------*/

/*CRC 2             */
Rf12_SendByte(0xF4);
/*------------------*/

/*CRC 3             */
Rf12_SendByte(crc3H);
Rf12_SendByte(crc3L);
/*------------------*/

/*Packet End        */
Rf12_SendByte(0xAA);
Rf12_SendByte(0xFF);
/*------------------*/
Write_Command(0x8208);
Write_Command(0x8200);
}